@@ -2,10 +2,10 @@ from __future__ import annotations
 import numpy as np
 from numpy.typing import ArrayLike
 from vispy import scene
-
-from .layer3d import Image, IsoSurface, Surface
+from .layer3d import Image, IsoSurface, Surface, Curve3D
 from .layerlist import LayerList
-from ._base import SceneCanvas, HasViewBox, MultiPlot
+from ._base import SceneCanvas, HasViewBox, MultiPlot, LayerItem
+from .camera import Camera
 
 from ...widgets import FreeWidget
 from ...types import Color
@@ -21,7 +21,7 @@ class Has3DViewBox(HasViewBox):
 
     def __init__(self, viewbox: scene.ViewBox):
         super().__init__(viewbox)
-        self._viewbox.camera = scene.ArcballCamera(fov=0)
+        self._camera = Camera(viewbox)
 
     @property
     def layers(self):
@@ -29,9 +29,9 @@ class Has3DViewBox(HasViewBox):
         return self._layerlist
 
     @property
-    def camera(self):
+    def camera(self) -> Camera:
         """Return the native camera."""
-        return self._viewbox.camera
+        return self._camera
 
     def add_image(
         self,
@@ -57,10 +57,7 @@ class Has3DViewBox(HasViewBox):
             interpolation=interpolation,
         )
 
-        self._layerlist.append(image)
-        self._viewbox.camera.scale_factor = max(data.shape)
-        self._viewbox.camera.center = [s / 2 - 0.5 for s in data.shape]
-        return image
+        return self.add_layer(image)
 
     def add_isosurface(
         self,
@@ -82,10 +79,7 @@ class Has3DViewBox(HasViewBox):
             shading=shading,
         )
 
-        self._layerlist.append(surface)
-        self._viewbox.camera.scale_factor = max(data.shape)
-        self._viewbox.camera.center = [s / 2 - 0.5 for s in data.shape]
-        return surface
+        return self.add_layer(surface)
 
     def add_surface(
         self,
@@ -102,12 +96,33 @@ class Has3DViewBox(HasViewBox):
             edge_color=edge_color,
             shading=shading,
         )
-        self._layerlist.append(surface)
-        mins = np.min(data[0], axis=0)
-        maxs = np.max(data[0], axis=0)
-        self._viewbox.camera.scale_factor = max(maxs - mins)
-        self._viewbox.camera.center = [(s1 + s0) / 2 for s0, s1 in zip(mins, maxs)]
-        return surface
+        return self.add_layer(surface)
+
+    def add_curve(
+        self,
+        data: ArrayLike,
+        color="white",
+        width=1,
+    ):
+        curve = Curve3D(
+            data=np.asarray(data, dtype=np.float32),
+            viewbox=self._viewbox,
+            color=color,
+            width=width,
+        )
+        return self.add_layer(curve)
+
+    def add_layer(self, layer: LayerItem):
+        """Add a layer item to the canvas."""
+        self.layers.append(layer)
+        if len(self.layers) == 1:
+            low, high = layer._get_bbox()
+            self.camera.scale = max(high - low)
+            self.camera.center = (high + low) / 2
+            self.camera.angles = (0.0, 0.0, 90.0)
+
+        self._viewbox.update()
+        return layer
 
 
 class Vispy3DCanvas(FreeWidget, Has3DViewBox):
