@@ -3,7 +3,6 @@ from enum import Enum
 import typing
 from typing import (
     Any,
-    NamedTuple,
     Union,
     Iterable,
     overload,
@@ -22,7 +21,16 @@ try:
 except ImportError:
     _tp_cache = lambda x: x
 
-__all__ = ["WidgetType", "bound", "Bound", "Choices", "OneOf", "SomeOf", "Optional"]
+__all__ = [
+    "WidgetType",
+    "bound",
+    "Bound",
+    "Choices",
+    "OneOf",
+    "SomeOf",
+    "Optional",
+    "Union",
+]
 
 
 class WidgetType(Enum):
@@ -43,64 +51,42 @@ class WidgetType(Enum):
     mainwindow = "mainwindow"
 
 
-WidgetTypeStr = Union[
-    Literal["none"],
-    Literal["scrollable"],
-    Literal["draggable"],
-    Literal["split"],
-    Literal["collapsible"],
-    Literal["button"],
-    Literal["toolbox"],
-    Literal["tabbed"],
-    Literal["stacked"],
-    Literal["list"],
-    Literal["subwindows"],
-    Literal["groupbox"],
-    Literal["frame"],
-    Literal["mainwindow"],
-    Literal["hcollapsible"],
+WidgetTypeStr = Literal[
+    "none",
+    "scrollable",
+    "draggable",
+    "split",
+    "collapsible",
+    "button",
+    "toolbox",
+    "tabbed",
+    "stacked",
+    "list",
+    "subwindows",
+    "groupbox",
+    "frame",
+    "mainwindow",
+    "hcollapsible",
 ]
 
 
-PopUpModeStr = Union[
-    Literal["popup"],
-    Literal["first"],
-    Literal["last"],
-    Literal["above"],
-    Literal["below"],
-    Literal["dock"],
-    Literal["dialog"],
-    Literal["parentlast"],
+PopUpModeStr = Literal[
+    "popup",
+    "first",
+    "last",
+    "above",
+    "below",
+    "dock",
+    "dialog",
+    "parentlast",
 ]
 
 
-ErrorModeStr = Union[
-    Literal["msgbox"],
-    Literal["stderr"],
-    Literal["stdout"],
-]
+ErrorModeStr = Literal["msgbox", "stderr", "stdout"]
 
 Color = Union[Iterable[float], str]
 
 
-class ColorArray(NamedTuple):
-    r: float
-    g: float
-    b: float
-    a: float
-
-    # def __eq__(self, other) -> bool:
-    #     if isinstance(other, str):
-    #         ...
-    #     return super().__eq__(other)
-
-    # def __str__(self) -> str:
-    #     code = "#" + "".join(hex(int(c * 255))[2:].upper().zfill(2) for c in self)
-    #     if code.endswith("FF"):
-    #         code = code[:-2]
-    #     return code
-
-
 _W = TypeVar("_W", bound=Widget)
 _V = TypeVar("_V", bound=object)
 
@@ -129,9 +115,27 @@ def bound(obj):
         raise TypeError("'bound' can only convert callable, MagicField or type objects")
     while isinstance(outtype, _AnnotatedAlias):
         outtype, _ = split_annotated_type(outtype)
+    if isinstance(obj, str):
+        obj = BoundLiteral(obj)
     return Annotated[outtype, {"bind": obj, "widget_type": EmptyWidget}]
 
 
+class BoundLiteral:
+    """
+    A class used to represent a future evaluable expression.
+
+    This object will be created when a string is passed to the ``Bound[...]`` type.
+    """
+
+    def __init__(self, expr: str):
+        self._expr = expr
+
+    def eval(self, cls: type) -> Any:
+        from .utils import eval_attribute
+
+        return eval_attribute(cls, self._expr)
+
+
 class _BoundAlias(type):
     """
     This metaclass is necessary for ``mypy`` to reveal type.
@@ -419,6 +423,50 @@ class Optional(metaclass=_OptionalAlias):
         raise TypeError(f"Cannot subclass {cls.__module__}.Optional.")
 
 
+class _UnionAlias(type):
+    def __getitem__(cls, value):
+        from .functools._dispatch import UnionWidget
+
+        annotations = []
+        opt = dict(widget_type=UnionWidget, nullable=False, annotations=annotations)
+        for val in value:
+            if val is not None:
+                annotations.append(val)
+            else:
+                opt["nullable"] = True
+        # union = Union[tuple(annotations)]  # type: ignore
+        return Annotated[Any, opt]
+
+
+class Union(metaclass=_UnionAlias):
+    """
+    Make Annotated type similar to ``typing.Union``.
+
+    Arguments annotated with ``Union[int, str]`` will create a
+    ``UnionWidget`` with a ``SpinBox`` and a ``LineEdit`` as inner widgets.
+
+    Examples
+    --------
+
+    from magicclass import magicclass
+    from magicclass.types import Union
+
+    @magicclass
+    class A:
+        def f(self, a: Union[int, str]):
+            print(a)
+
+    ui = A()
+    ui.show()
+    """
+
+    def __new__(cls, *args, **kwargs):
+        raise TypeError("Type Union cannot be instantiated.")
+
+    def __init_subclass__(cls, *args, **kwargs):
+        raise TypeError(f"Cannot subclass {cls.__module__}.Union.")
+
+
 def __getattr__(key: str):
     if key in ["List", "Tuple"]:
         import warnings
