@@ -513,6 +513,10 @@ class thread_worker:
         """True if bound function is a generator function."""
         return inspect.isgeneratorfunction(self._func)
 
+    @property
+    def __is_recordable__(self) -> bool:
+        return self._recorder is not None
+
     @property
     def __doc__(self) -> str:
         """Synchronize docstring with bound function."""
