@@ -378,6 +378,13 @@ class FloatRangeSlider(AbstractRangeSlider[float]):
         return sl
 
 
+# magicgui>=0.6.0 has its own range sliders.
+try:
+    from magicgui.widgets import RangeSlider, FloatRangeSlider
+except ImportError:
+    pass
+
+
 class _QtSpreadSheet(QtW.QTabWidget):
     def __init__(self):
         super().__init__()
@@ -530,9 +537,13 @@ class _QEditableComboBox(QtW.QComboBox):
 class QHistoryLineEdit(QBaseStringWidget):
     _qwidget: _QEditableComboBox
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         super().__init__(
-            _QEditableComboBox, "currentText", "setCurrentText", "currentTextChanged"
+            _QEditableComboBox,
+            "currentText",
+            "setCurrentText",
+            "currentTextChanged",
+            **kwargs,
         )
 
 
